package com.example.thomasbet;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Typeface;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import android.os.Handler;
import android.os.Looper;
import android.text.Layout;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.style.AlignmentSpan;
import android.text.style.ForegroundColorSpan;
import android.text.style.StyleSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;
import android.widget.Toast;

import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.customDataTypes.Tuple;
import com.example.thomasbet.model.UserEntity;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import at.favre.lib.crypto.bcrypt.BCrypt;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link AdminViewFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class AdminViewFragment extends Fragment {

    private CheckBox emailCheckbox;
    private CheckBox nicknameCheckbox;
    private EditText searchText;
    private TableLayout tableLayout;

    private List<UserEntity> userSearchResult = new ArrayList<UserEntity>();

    private List<Tuple<List<CheckBox>, UserEntity>> tupleList = new ArrayList<>();

    public AdminViewFragment() {
    }

    public static AdminViewFragment newInstance() {
        AdminViewFragment fragment = new AdminViewFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
    }

    @SuppressLint("MissingInflatedId")
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_admin_view, container, false);


        this.tableLayout = view.findViewById(R.id.table_adminView);

        this.emailCheckbox = view.findViewById(R.id.checkBox);
        this.nicknameCheckbox = view.findViewById(R.id.checkBox2);
        this.searchText = view.findViewById(R.id.userFilter);
        Button searchButton = view.findViewById(R.id.searchButton);
        Button saveButton = view.findViewById(R.id.saveButton);

        nicknameCheckbox.setOnCheckedChangeListener((checkbox, value) -> {
            if(value) emailCheckbox.setChecked(false);
        });

        emailCheckbox.setOnCheckedChangeListener((checkbox, value) -> {
            if(value) nicknameCheckbox.setChecked(false);
        });

        searchButton.setOnClickListener((button) -> {
            this.tableLayout.removeAllViews();
            try {
                processInput();
            } catch (IOException e) {
                Toast.makeText(getActivity(), "Uživatel nebyl nalezen", Toast.LENGTH_SHORT).show();
            }
        });



        saveButton.setOnClickListener((button) -> {
            saveRoles(this.userSearchResult);
        });

        return view;
    }



    //vrácení uživatelova emailu nebo nicknamu, podle hledaného parametru
    private void processInput() throws IOException {
        String emailValue = emailCheckbox.isChecked() ? searchText.getText().toString() : null;
        String nicknameValue = nicknameCheckbox.isChecked() ? searchText.getText().toString() : null;


        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());
        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    userSearchResult = UserRestClient.GET_USERS_BY_EMAIL_OR_NICKNAME(emailValue, nicknameValue);
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            setColumnHeaders(emailValue, nicknameValue);
                            if(userSearchResult != null){
                                userSearchResult.forEach(user -> {
                                    addUsersTableRow(user, emailValue, nicknameValue);
                                });
                            }else{

                            }
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });



    }

    //dynamické přidání hlavičky tabulky včetně stylizace
    private void setColumnHeaders(String email, String nickname){
        TableRow row = new TableRow(getContext());

        //Nickname
        TextView nicknameColumn = new TextView(getContext());
        SpannableStringBuilder spannableNickname = new SpannableStringBuilder("Nickname");

        spannableNickname.setSpan(new StyleSpan(Typeface.BOLD), 0, "Nickname".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
        ForegroundColorSpan colorSpan = new ForegroundColorSpan(Color.GREEN);
        spannableNickname.setSpan(colorSpan, 0, "Nickname".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        int textLength = spannableNickname.length();
        AlignmentSpan.Standard alignmentSpan = new AlignmentSpan.Standard(Layout.Alignment.ALIGN_CENTER);
        spannableNickname.setSpan(alignmentSpan, 0, textLength, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        nicknameColumn.setText(spannableNickname);

        //Email
        TextView emailColumn = new TextView(getContext());
        SpannableStringBuilder spannableEmail = new SpannableStringBuilder("Email");
        spannableEmail.setSpan(new StyleSpan(Typeface.BOLD), 0, "Email".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        ForegroundColorSpan colorSpanEmail = new ForegroundColorSpan(Color.GREEN);
        spannableEmail.setSpan(colorSpanEmail, 0, "Email".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        int textLengthEmail = spannableEmail.length();
        AlignmentSpan.Standard alignmentSpanEmail = new AlignmentSpan.Standard(Layout.Alignment.ALIGN_CENTER);
        spannableEmail.setSpan(alignmentSpanEmail, 0, textLengthEmail, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        emailColumn.setText(spannableEmail);

        //Admin
        TextView adminColumn = new TextView(getContext());
        SpannableStringBuilder spannableAdmin = new SpannableStringBuilder("Admin");
        spannableAdmin.setSpan(new StyleSpan(Typeface.BOLD), 0,"Admin".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        ForegroundColorSpan colorSpanAdmin= new ForegroundColorSpan(Color.GREEN);
        spannableAdmin.setSpan(colorSpanAdmin, 0, "Admin".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        int textLengthAdmin = spannableAdmin.length();
        AlignmentSpan.Standard alignmentSpanAdmin = new AlignmentSpan.Standard(Layout.Alignment.ALIGN_CENTER);
        spannableAdmin.setSpan(alignmentSpanAdmin, 0, textLengthAdmin, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        adminColumn.setText(spannableAdmin);

        //Better
        TextView betterColumn = new TextView(getContext());
        SpannableStringBuilder spannableBetter = new SpannableStringBuilder("Better");
        spannableBetter.setSpan(new StyleSpan(Typeface.BOLD), 0,"Better".length(), 0);

        ForegroundColorSpan colorSpanBetter= new ForegroundColorSpan(Color.GREEN);
        spannableBetter.setSpan(colorSpanBetter, 0, "Better".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        int textLengthBetter = spannableBetter.length();
        AlignmentSpan.Standard alignmentSpanBetter = new AlignmentSpan.Standard(Layout.Alignment.ALIGN_CENTER);
        spannableBetter.setSpan(alignmentSpanBetter, 0, textLengthBetter, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        betterColumn.setText(spannableBetter);

        //User
        TextView userColumn = new TextView(getContext());
        SpannableStringBuilder spannableUser = new SpannableStringBuilder("User");
        spannableUser.setSpan(new StyleSpan(Typeface.BOLD), 0,"User".length(), 0);

        ForegroundColorSpan colorSpanUser= new ForegroundColorSpan(Color.GREEN);
        spannableUser.setSpan(colorSpanUser, 0, "User".length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        int textLengthUser = spannableUser.length();
        AlignmentSpan.Standard alignmentSpanUser = new AlignmentSpan.Standard(Layout.Alignment.ALIGN_CENTER);
        spannableUser.setSpan(alignmentSpanUser, 0, textLengthUser, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);

        userColumn.setText(spannableUser);

        if(nickname == null){
            row.addView(emailColumn);row.addView(adminColumn);row.addView(betterColumn);row.addView(userColumn);
        }else{
            row.addView(nicknameColumn);row.addView(adminColumn);row.addView(betterColumn);row.addView(userColumn);
        }


        this.tableLayout.addView(row);

    }


    //dynamické přidání řádků do tabulky včetně stylizace
    private void addUsersTableRow(UserEntity user, String email, String nickname){
        TableRow row = new TableRow(getContext());
        TextView resultColumn = new TextView(getContext());
        if(email == null){
            resultColumn.setText(user.getNickname());
        }else{
            resultColumn.setText(user.getEmail());
        }


        List<CheckBox> checkboxes = new ArrayList<CheckBox>();

        CheckBox adminRoleCheckbox = new CheckBox(getContext());
        if(user.getRole_id() == 1) adminRoleCheckbox.setChecked(true);
        adminRoleCheckbox.setHint("admin");
        adminRoleCheckbox.setOnCheckedChangeListener((checkbox, isChecked) -> {
            uncheckOtherCheckboxes(adminRoleCheckbox, isChecked, user);
        });
        checkboxes.add(adminRoleCheckbox);


        CheckBox betterRoleCheckbox = new CheckBox(getContext());
        if(user.getRole_id() == 2) betterRoleCheckbox.setChecked(true);
        betterRoleCheckbox.setHint("better");
        betterRoleCheckbox.setOnCheckedChangeListener((checkbox, isChecked) -> {
            uncheckOtherCheckboxes(betterRoleCheckbox, isChecked, user);
        });
        checkboxes.add(betterRoleCheckbox);

        CheckBox userRoleCheckbox = new CheckBox(getContext());
        if(user.getRole_id() == 3) userRoleCheckbox.setChecked(true);
        userRoleCheckbox.setHint("user");
        userRoleCheckbox.setOnCheckedChangeListener((checkbox, isChecked) -> {
            uncheckOtherCheckboxes(userRoleCheckbox, isChecked, user);
        });
        checkboxes.add(userRoleCheckbox);

        tupleList.add(new Tuple(checkboxes, user));


        row.addView(resultColumn);row.addView(adminRoleCheckbox);row.addView(betterRoleCheckbox);row.addView(userRoleCheckbox);

        this.tableLayout.addView(row);
    }

    //odškrnutí checkboxu při zaškrnutí jiného
    private void uncheckOtherCheckboxes(CheckBox checkBox, Boolean isChecked, UserEntity user){
        tupleList.forEach((tuple) -> {
            if(tuple.getSecond().equals(user)){
                tuple.getFirst().forEach((box) -> {
                    if(!box.equals(checkBox) && isChecked){
                        if(box.isChecked()){
                            box.setChecked(false);
                        }
                    }
                });
            }

            if(isChecked){
                userSearchResult.forEach((searchedUser) -> {
                    if(searchedUser.equals(user)){
                        switch (checkBox.getHint().toString()){
                            case "user":
                                user.setRole_id(2);
                                break;
                            case "better":
                                user.setRole_id(3);
                                break;
                            case "admin":
                                user.setRole_id(1);
                                break;
                        }
                    }
                });
            }
        });
    }


    //uloží změny rolí
    private void saveRoles(List<UserEntity> users){

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                    users.forEach(user -> {
                        try{
                            UserRestClient.UPDATE_USER(user);
                        }catch(Exception E){
                            System.out.println(E);
                        }
                    });

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(getActivity(), "Role byly upraveny", Toast.LENGTH_SHORT).show();
                        }
                    });
            }
        });
    }
}