package com.example.thomasbet;

import android.os.Bundle;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import com.example.thomasbet.RestClients.EventRestClient;
import com.example.thomasbet.customDataTypes.event.AdapterEventUpcoming;

import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class AllMatchesFragment extends Fragment implements AdapterView.OnItemSelectedListener {
    Spinner matchesCategory;

    private MainActivity mainActivity;
    private RecyclerView events;
    private Button eventLoadMore_btn;
    private Button eventLoadless_btn;
    private AdapterEventUpcoming adapterEvent;
    private int currentPage;
    private int currentOffset = 2;
    private int pagesCount;

    private String sportType;

    public AllMatchesFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_all_matches, container, false);

        matchesCategory = view.findViewById(R.id.allmatches_allsports);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(getContext(),R.array.spinner_SportCategories, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        matchesCategory.setAdapter(adapter);

        matchesCategory.setOnItemSelectedListener(this);

        this.adapterEvent = new AdapterEventUpcoming(getContext(), (MainActivity) getActivity(), this);
        mainActivity = (MainActivity) getActivity();
        sportType = "all";

        events = view.findViewById(R.id.allmatches);
        eventLoadMore_btn = view.findViewById(R.id.allmatches_loadmore_btn);
        eventLoadless_btn = view.findViewById(R.id.allmatches_loadless_btn);

        eventLoadMore_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(currentPage < pagesCount -1){
                    currentPage++;
                    updateButtons();
                    loadEvents(currentOffset, currentPage, sportType);
                }else{
                    eventLoadMore_btn.setVisibility(View.GONE);
                }
            }
        });
        eventLoadless_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                currentPage--;
                updateButtons();
                loadEvents(currentOffset, currentPage, sportType);
                if (currentPage == 0) {
                    eventLoadless_btn.setVisibility(View.GONE);
                }
            }
        });

        if (currentPage == 0) {
            eventLoadless_btn.setVisibility(View.GONE);
        }

        if(currentPage == pagesCount-1){
            eventLoadMore_btn.setVisibility(View.GONE);
        }

        return view;
    }


    //při vybrání možnosti ze spinneru, zobrazení správných dat
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        switch(parent.getItemAtPosition(position).toString()) {
            case "Všechno":
                this.sportType = "all";
                break;
            case "Fotbal":
                this.sportType = "football";
                break;
            case "Hokej":
                this.sportType = "ice_hockey";
                break;
        }

        loadEvents(currentOffset, currentPage, sportType);
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {
        //loadEvents(currentOffset, currentPage, "all");
    }

    //načtení dat
    private void loadEvents(int offset, int page, String sportType){
        this.adapterEvent.removeModels();

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    pagesCount = EventRestClient.GET_PAGES_COUNT(offset, sportType, "upcoming");
                    Objects.requireNonNull(EventRestClient.GET_UPCOMING_EVENTS_PAGED(offset, page, sportType)).forEach(model -> {
                            adapterEvent.addModelEvent(model);
                    });

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            events.setAdapter(adapterEvent);
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });
    }

    //funkcionalita tlačítek podle vráceného stránkování a počtu vrácených příspěvků
    private void updateButtons(){
        if(currentPage == 0){
            eventLoadless_btn.setVisibility(View.GONE);
        }else{
            eventLoadless_btn.setVisibility(View.VISIBLE);
        }

        if(currentPage < pagesCount - 1){
            eventLoadMore_btn.setVisibility(View.VISIBLE);
        }else{
            eventLoadMore_btn.setVisibility(View.GONE);
        }
    }
}