package com.example.thomasbet;

import android.os.Bundle;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import com.example.thomasbet.Enums.UserRoles;
import com.example.thomasbet.RestClients.BlogRestClient;
import com.example.thomasbet.RestClients.StrategyRestClient;
import com.example.thomasbet.customDataTypes.blog.AdapterBlog;
import com.example.thomasbet.customDataTypes.strategy.AdapterStrategy;

import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class BlogFragment extends Fragment {
    Button AddArticleButton;
    Button addStrategyButton;
    Button Strategie;
    Button ProfiBetters;
    private MainActivity mainActivity;


    private RecyclerView blogPosts;
    private Button blogLoadMore_btn;
    private Button blogLoadless_btn;
    private AdapterBlog adapterBlog;
    private int currentPage;
    private int currentOffset = 2;
    private int pagesCount;
    private Long userIdFilter = 0L;

    public BlogFragment() {
        this.currentPage = 0;
    }

    public BlogFragment(int page, Long userIdFilter) {
        this.currentPage = page;
        this.userIdFilter = userIdFilter;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_blog, container, false);

        this.adapterBlog = new AdapterBlog(getContext(), (MainActivity) getActivity(), this);
        AddArticleButton = view.findViewById(R.id.blog_addArticle_button);
        addStrategyButton = view.findViewById(R.id.blog_addStrategy_button);
        mainActivity = (MainActivity) getActivity();

        blogPosts = view.findViewById(R.id.blog_posts);
        blogLoadMore_btn = view.findViewById(R.id.blog_loadmore_btn);
        blogLoadless_btn = view.findViewById(R.id.blog_loadless_btn);

        loadposts(currentOffset, currentPage);

        blogLoadMore_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(currentPage < pagesCount -1){
                    currentPage++;
                    updateButtons();
                    loadposts(currentOffset, currentPage);
                }else{
                    blogLoadMore_btn.setVisibility(View.GONE);
                }
            }
        });
        blogLoadless_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                currentPage--;
                updateButtons();
                loadposts(currentOffset, currentPage);
                if (currentPage == 0) {
                    blogLoadless_btn.setVisibility(View.GONE);
                }
            }
        });

        if (currentPage == 0) {
            blogLoadless_btn.setVisibility(View.GONE);
        }

        if(currentPage == pagesCount-1){
            blogLoadMore_btn.setVisibility(View.GONE);
        }

        if(mainActivity.getUser().getRole_id() == UserRoles.ADMIN.getValue() || mainActivity.getUser().getRole_id() == UserRoles.BETTER.getValue() ){
            AddArticleButton.setVisibility(View.VISIBLE);
            addStrategyButton.setVisibility(View.VISIBLE);

            AddArticleButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    MainActivity mainActivity = (MainActivity) getActivity();
                    mainActivity.changeFragment(new BlogFragmentNewArticle());
                }
            });

            addStrategyButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    MainActivity mainActivity = (MainActivity) getActivity();
                    mainActivity.changeFragment(new StrategyFragmentNewArticle());
                }
            });
        }else{
            AddArticleButton.setVisibility(View.GONE);
            addStrategyButton.setVisibility(View.GONE);
        }





        Strategie = view.findViewById(R.id.blog_strategy);
        Strategie.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                MainActivity mainActivity = (MainActivity) getActivity();
                mainActivity.changeFragment(new StrategyFragment());
            }
        });

        return view;
    }




    private void loadposts(int offset, int page){
        // smaže zobrazené modely
        this.adapterBlog.removeModels();

        // přesunutí akce do pozadí, nesmí běžet na hlavním vlákně UI
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {

                    // userIDFilter je z důvodu rozeznání, zdali profi-sázkař zobrazuje jen své strategie nebo všechny
                    if(userIdFilter == 0){
                        pagesCount = BlogRestClient.GET_PAGES_COUNT(offset, 0L, false);
                        Objects.requireNonNull(BlogRestClient.GET_BLOGS_PAGED(offset, page)).forEach(model -> {
                            adapterBlog.addModelBlog(model);
                        });
                    }else{
                        pagesCount = BlogRestClient.GET_PAGES_COUNT(offset, userIdFilter, true);
                        Objects.requireNonNull(BlogRestClient.GET_BLOGS_BY_USER_ID(userIdFilter, page, offset)).forEach(model -> {
                            adapterBlog.addModelBlog(model);
                        });
                    }

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            blogPosts.setAdapter(adapterBlog);
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });
    }
    //funkcionalita tlačítek podle vráceného stránkování a počtu vrácených příspěvků
    private void updateButtons(){
        if(currentPage == 0){
            blogLoadless_btn.setVisibility(View.GONE);
        }else{
            blogLoadless_btn.setVisibility(View.VISIBLE);
        }

        if(currentPage < pagesCount - 1){
            blogLoadMore_btn.setVisibility(View.VISIBLE);
        }else{
            blogLoadMore_btn.setVisibility(View.GONE);
        }
    }

    //smazání článku
    public void deleteBlog(Long id) {
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    BlogRestClient.DELETE_BLOG(id);
                    handler.post(new Runnable() {
                        @Override
                        public void run() {

                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });

        loadposts(currentOffset, currentPage);
    }


    //refresh stránky pro aktualizaci změn
    public void refresh(){
        loadposts(currentOffset, currentPage);
    }
}