package com.example.thomasbet;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;


import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.model.UserEntity;

import java.io.IOException;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import at.favre.lib.crypto.bcrypt.BCrypt;

public class LoginFragment extends Fragment {

    private EditText emailField;
    private EditText passwordField;
    private TextView registerText;
    private Button loginButton;
    private EditText ShowPassword_login;
    private List<OnLoginSuccessListener> loginSuccessListeners = new ArrayList<>();
    private OnRegisterClickListener registerClickListener;

    public interface OnLoginSuccessListener {
        void onLoginSuccess(UserEntity user);
    }

    public interface OnRegisterClickListener {
        void moveToRegisterFragment();
    }
    public LoginFragment() {
        // Required empty public constructor
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_login, container, false);
        emailField = view.findViewById(R.id.email_login);
        passwordField = view.findViewById(R.id.password_login);
        loginButton = view.findViewById(R.id.btn_login);
        registerText = view.findViewById(R.id.register_redirect_text);

        registerText.setOnClickListener((text) -> {
            registerClickListener.moveToRegisterFragment();
        });

        loginButton.setOnClickListener((button) -> {
            try {
                loginUser(emailField.getText().toString(),
                        passwordField.getText().toString());
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        });

        return view;

    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);

        registerClickListener = (OnRegisterClickListener) context;
        if (context instanceof OnLoginSuccessListener) {
            loginSuccessListeners.add((OnLoginSuccessListener) context);
        } else {
            throw new RuntimeException(context.toString() + " must implement OnLoginSuccessListener");
        }
    }


    //přihlášení uživatele
    private void loginUser(String email, String password) throws IOException {
        // přesunutí akce do pozadí, nesmí běžet na hlavním vlákně UI
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                UserEntity user;
                try {
                    // zavolání RestClient požadavku
                    user = UserRestClient.GET_USER_AFTER_LOGIN(email);

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            if(user != null){
                                // porovnání zadaného hesla s DB
                                if(BCrypt.verifyer().verify(passwordField.getText().toString().toCharArray(), user.getPassword()).verified){
                                    notifyListeners(user);
                                    Toast.makeText(getActivity(), "Přihlášení úspěšné", Toast.LENGTH_SHORT).show();
                                }else{
                                    Toast.makeText(getActivity(), "Špatné heslo", Toast.LENGTH_SHORT).show();
                                }
                            }else{
                                // ošetření, pokud uživatel zadá špatně email
                                Toast.makeText(getActivity(), "Účet s tímto emailem neexistuje", Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
                } catch (Exception e) {
                    // do této sekce by se uživatel neměl nikdy dostat
                    System.out.println(e);
                }
            }
        });
    }

    public void addLoginSuccessListener(OnLoginSuccessListener listener){
        loginSuccessListeners.add(listener);
    }

    private void notifyListeners(UserEntity user){
        loginSuccessListeners.forEach((listener) -> {
            listener.onLoginSuccess(user);
        });
    }
}