package com.example.thomasbet;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.widget.Toolbar;

import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.interfaces.UserUpdateListener;
import com.example.thomasbet.model.UserEntity;
import com.google.android.material.internal.NavigationMenu;
import com.google.android.material.navigation.NavigationView;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class MainActivity extends AppCompatActivity implements NavigationView.OnNavigationItemSelectedListener, LoginFragment.OnLoginSuccessListener, LoginFragment.OnRegisterClickListener, RegisterFragment.OnLoginClickListener, RegisterFragment.OnRegisterSuccessListener, UserUpdateListener {

    //předělat
    static int PROFILE_BUTTON_INDEX = 5;
    static int LOGIN_BUTTON_INDEX = 6;
    static int REGISTER_BUTTON_INDEX = 7;
    static int LOGOUT_BUTTON_INDEX = 8;
    static int ADMINVIEW_BUTTON_INDEX = 9;
    static int STRATEGY_BUTTON_INDEX = 2;
    static int BLOG_BUTTON_INDEX = 1;
    static int MATCHES_BUTTON_INDEX = 3;
    static int HOME_BUTTON_INDEX = 0;
    static int RESULTS_BUTTON_INDEX = 4;

    private UserEntity loggedUser;
    private final List<UserUpdateListener> listeners = new ArrayList<UserUpdateListener>();
    ImageView profileIcon;
    ImageView homeIcon;




    //pro menu
    DrawerLayout drawerLayout;
    FragmentManager fragmentManager;
    Toolbar toolbar;

    private NavigationView navigationView;
    private Menu navMenu;

    private TextView balance;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setTitle("");
        setContentView(R.layout.activity_main);

        toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        drawerLayout = findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(this, drawerLayout, toolbar, R.string.open_nav, R.string.close_nav);
        drawerLayout.addDrawerListener(toggle);
        toggle.syncState();
        toolbar.setNavigationIcon(R.drawable.ic_menu);

        this.navigationView = findViewById(R.id.nav_drawer);
        navigationView.setNavigationItemSelectedListener(this);
        this.navMenu = navigationView.getMenu();

        navMenu.getItem(LOGOUT_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(PROFILE_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(ADMINVIEW_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(STRATEGY_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(BLOG_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(MATCHES_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(RESULTS_BUTTON_INDEX).setVisible(false);


        findMenuItemByTitle();

        fragmentManager = getSupportFragmentManager();
        openFragment(new HomeFragment());

        homeIcon = findViewById(R.id.logoHome);
        homeIcon.setOnClickListener((click) ->{
            changeFragment(new HomeFragment());
        });

        this.balance = findViewById(R.id.balanceToolbar);

        addListener(this);
        balance.setVisibility(View.INVISIBLE);
        ImageView profileIcon = findViewById(R.id.profile_icon);
        profileIcon.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v){
                changeFragment(new RegisterFragment());
            }
        });


    }



    //metoda pro otevírání fragmentů
    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem item) {
        int itemID = item.getItemId();
        if (itemID == R.id.nav_home)
            openFragment(new HomeFragment());
        else if (itemID == R.id.nav_blog) {
            openFragment(new BlogFragment());
        } else if (itemID == R.id.nav_strategy) {
            openFragment(new StrategyFragment());
        } else if (itemID == R.id.nav_login) {
            openFragment(new LoginFragment());
        } else if (itemID == R.id.nav_profile) {
            openFragment(new ProfileFragmanet());
        } else if (itemID == R.id.nav_register) {
            openFragment(new RegisterFragment());
        } else if (itemID == R.id.nav_results) {
            openFragment(new MatchResultsFragment());
        } else if (itemID == R.id.nav_adminView) {
            openFragment(new AdminViewFragment());
        } else if (itemID == R.id.nav_matches) {
            openFragment(new AllMatchesFragment());
        } else if (itemID == R.id.nav_logout){
            navMenu.getItem(REGISTER_BUTTON_INDEX).setVisible(true);
            navMenu.getItem(LOGIN_BUTTON_INDEX).setVisible(true);
            navMenu.getItem(LOGOUT_BUTTON_INDEX).setVisible(false);
            navMenu.getItem(PROFILE_BUTTON_INDEX).setVisible(false);
            navMenu.getItem(ADMINVIEW_BUTTON_INDEX).setVisible(false);
            navMenu.getItem(STRATEGY_BUTTON_INDEX).setVisible(false);
            navMenu.getItem(BLOG_BUTTON_INDEX).setVisible(false);
            navMenu.getItem(MATCHES_BUTTON_INDEX).setVisible(false);
            navMenu.getItem(RESULTS_BUTTON_INDEX).setVisible(false);
            ImageView profileIcon = findViewById(R.id.profile_icon);
            profileIcon.setImageResource(R.drawable.menu_login_icon);
            setUser(null);
            profileIcon.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v){
                    changeFragment(new RegisterFragment());
                }
            });
            openFragment(new HomeFragment());
        }
        drawerLayout.closeDrawer((GravityCompat.START));
        return true;
    }


    // fungování zpětného tlačítka z menu
    @Override
    public void onBackPressed() {
        if (drawerLayout.isDrawerOpen(GravityCompat.START)){
            drawerLayout.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    // nahrazování fragmentů
    private void openFragment(Fragment fragment){
        FragmentTransaction transaction = fragmentManager.beginTransaction();
        transaction.replace(R.id.fragment_container, fragment);
        transaction.commit();
    }

    @Override
    public void onLoginSuccess(UserEntity user) {
        navMenu.getItem(REGISTER_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(LOGIN_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(LOGOUT_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(PROFILE_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(ADMINVIEW_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(STRATEGY_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(BLOG_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(MATCHES_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(HOME_BUTTON_INDEX).setVisible(false);
        navMenu.getItem(MATCHES_BUTTON_INDEX).setVisible(true);
        navMenu.getItem(RESULTS_BUTTON_INDEX).setVisible(true);

        setUser(user);

        balance.setVisibility(View.VISIBLE);
        balance.setText(String.valueOf(this.loggedUser.getBalance()));

        getSupportFragmentManager().beginTransaction()
                .replace(R.id.fragment_container, new ProfileFragmanet())
                .commit();


        this.profileIcon = findViewById(R.id.profile_icon);

        //Pokud se přihlásí sázkař nebo uživatel, bude se moct dostat na profil, pokud se přihlásí admin, přenese ho to na AdminPanel
        if (this.loggedUser.getRole_id() == 2 || this.loggedUser.getRole_id() == 3){
            navMenu.getItem(ADMINVIEW_BUTTON_INDEX).setVisible(false);
            profileIcon.setImageResource(R.drawable.icon_account_logo);
            profileIcon.setOnClickListener((click) -> {
                changeFragment(new ProfileFragmanet());
            });
        } else if (this.loggedUser.getRole_id() == 1) {
            navMenu.getItem(ADMINVIEW_BUTTON_INDEX).setVisible(true);
            TextView balanceToolbar = findViewById(R.id.balanceToolbar);
            balanceToolbar.setVisibility(View.INVISIBLE);
            navMenu.getItem(PROFILE_BUTTON_INDEX).setVisible(false);
            profileIcon.setImageResource(R.drawable.ic_options);
            profileIcon.setOnClickListener((click) -> {
                changeFragment(new AdminViewFragment());
            });
        }



    }

    @Override
    public void moveToRegisterFragment() {
        getSupportFragmentManager().beginTransaction()
                .replace(R.id.fragment_container, new RegisterFragment())
                .commit();
    }

    @Override
    public void onRegisterSuccess() {
        getSupportFragmentManager().beginTransaction()
                .replace(R.id.fragment_container, new HomeFragment())
                .commit();
    }

    @Override
    public void moveToLoginFragment() {
        getSupportFragmentManager().beginTransaction()
                .replace(R.id.fragment_container, new LoginFragment())
                .commit();
    }



    public void changeFragment(Fragment targetFragment){
        getSupportFragmentManager().beginTransaction()
                .replace(R.id.fragment_container, targetFragment)
                .commit();
    }

    public void updateBalance(int change){
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            UserEntity updatedUser;
            @Override
            public void run() {
                try {
                    updatedUser = UserRestClient.UPDATE_USER_BALANCE(loggedUser, change);
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            setUser(updatedUser);
                            balance.setText(String.valueOf(loggedUser.getBalance()));
                            notifyListeners();
                        }
                    });
                } catch (Exception e) {
                    Toast.makeText(getActivity(), "Něco se pokazilo, zkuste to prosím později", Toast.LENGTH_SHORT).show();
                }
            }
        });
    }

    private Activity getActivity(){
        return this;
    }
    public void addListener(UserUpdateListener listener){
        this.listeners.add(listener);
    }

    //upozornění listenerů při změně
    public void notifyListeners(){
        if(this.listeners != null){
            for(UserUpdateListener listener : listeners){
                listener.onUserUpdate(loggedUser);
            }
        }
    }

    public UserEntity getUser(){
        return loggedUser;
    }

    public void setUser(UserEntity user){
        this.loggedUser = user;
        notifyListeners();
    }

    //změna balancu
    @Override
    public void onUserUpdate(UserEntity user) {
        if(user != null){
            balance.setText(String.valueOf(loggedUser.getBalance()));
        }else{
            balance.setVisibility(View.INVISIBLE);
        }
    }

    private MenuItem findMenuItemByTitle (){
        for (int i = 0; i< navMenu.size(); i++){
            MenuItem item = navMenu.getItem(i);
            item.getTitle().toString();
            System.out.println(item);
        }
         return null;
    }





}