package com.example.thomasbet;

import android.graphics.Color;
import android.os.Bundle;

import androidx.fragment.app.Fragment;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TableLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.example.thomasbet.Enums.BetTypes;
import com.example.thomasbet.RestClients.BetRestClient;
import com.example.thomasbet.RestClients.EventRestClient;
import com.example.thomasbet.customDataTypes.betting.Bet;
import com.example.thomasbet.customDataTypes.event.AdapterEventUpcoming;
import com.example.thomasbet.customDataTypes.event.ModelEvent;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;


public class MatchDetailsFragment extends Fragment implements AdapterView.OnItemSelectedListener {

    private String eventId;
    private MainActivity mainActivity;
    private AdapterEventUpcoming.HolderEventUpcoming holderEvent;
    private BetTypes betTypes;
    private Float selectedOdd;
    private List<TextView> textViews = new ArrayList<>();

    private TextView category, commenceDate, title1,title2, league;


    private TableLayout tableLayout;

    private Button betButton;

    private EditText betValue;

    private Spinner betType;

    public MatchDetailsFragment(AdapterEventUpcoming.HolderEventUpcoming holderEvent, String eventId) {
        this.holderEvent = holderEvent;
        this.eventId = eventId;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_match_details, container, false);

        this.title1 = view.findViewById(R.id.matchDetails_title1);
        this.title2 = view.findViewById(R.id.matchDetails_title2);
        this.category = view.findViewById(R.id.matchDetails_category);
        this.league = view.findViewById(R.id.matchDetails_league);
        this.commenceDate = view.findViewById(R.id.matchDetails_playdate);

        mainActivity = (MainActivity) getActivity();

        this.category.setText(holderEvent.getCategory().getText());
        this.title1.setText(holderEvent.getTitle1().getText());
        this.title2.setText(holderEvent.getTitle2().getText());
        this.league.setText(holderEvent.getLeague().getText());
        this.commenceDate.setText(holderEvent.getCommenceDate().getText());
        this.tableLayout = view.findViewById(R.id.matchDetails_betTable);
        betValue = view.findViewById(R.id.matchDetails_betValue);
        betButton = view.findViewById(R.id.matchDetails_Bet_btn);
        betType = view.findViewById(R.id.matchDetails_betType);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(getContext(),R.array.spinner_betType, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        betType.setAdapter(adapter);


        betButton.setOnClickListener(click -> {
            try {
                realizeBet();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        });

        betType.setOnItemSelectedListener(this);

        return view;
    }

    //funkčnost spinneru, definování vykreslování tabulky
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        switch(parent.getItemAtPosition(position).toString()) {
            case "Výsledek zápasu":
                showMatchResultBetType();
                break;
            case "Výsledek zápasu bez remízy":
                showMatchResultWithoutDrawBetType();
                break;
            case "Dvojitá šance":
                showDoubleChanceBetType();
                break;
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {

    }


    // generátor kurzů
    private Float getRandomFloatNumber(){
        Random random = new Random();
        float randomValue = 1.1f + random.nextFloat() * (4.0f - 1.1f);
        return Math.round(randomValue * 100.0f) / 100.0f;
    }

    // při výběru sázky pole zezelená
    private void markSelectedView(TextView view){
        textViews.forEach(viewFromList -> {
            if(viewFromList.equals(view)){
                viewFromList.setBackgroundColor(Color.parseColor("#99cc00"));
            }else{
                viewFromList.setBackgroundColor(Color.GRAY);
            }
        });
    }

    //extrahování hodnoty kurzu pro pozdější uložení do DB
    private void extractFloatValue(TextView textView){
        String[] split = textView.getText().toString().split(" ");
        selectedOdd = Float.valueOf(split[split.length-1]);
    }

    //sázka na výsledek zápasu
    private void showMatchResultBetType(){
        tableLayout.removeAllViews();
        textViews.clear();
        LayoutInflater inflater = LayoutInflater.from(getActivity());
        // layout kategorie sázek
        View tableRow = inflater.inflate(R.layout.matchdetails_result, null);

        //Inicializace a výpis tabulky dostupných sázek včetně kurzu
        TextView home, draw, away;
        home = tableRow.findViewById(R.id.result_homeTeam);
        home.setText(home.getText() + " " + String.valueOf(getRandomFloatNumber()));

        draw = tableRow.findViewById(R.id.result_Draw);
        draw.setText(draw.getText() + " " + String.valueOf(getRandomFloatNumber()));

        away = tableRow.findViewById(R.id.result_awayTeam);
        away.setText(away.getText() + " " + String.valueOf(getRandomFloatNumber()));

        textViews.add(home);
        textViews.add(draw);
        textViews.add(away);

        //listenery naslouchající, zdali na ně uživatel kliknul
        home.setOnClickListener(click -> {
            this.betTypes = BetTypes.HOME_WIN;
            markSelectedView(home);
            extractFloatValue(home);
        });
        draw.setOnClickListener(click -> {
            this.betTypes = BetTypes.DRAW;
            markSelectedView(draw);
            extractFloatValue(draw);
        });
        away.setOnClickListener(click -> {
            this.betTypes = BetTypes.AWAY_WIN;
            markSelectedView(away);
            extractFloatValue(away);
        });

        tableLayout.addView(tableRow);
    }


    //sázka na výsledek zápasu bez remízy
    private void showMatchResultWithoutDrawBetType(){
        tableLayout.removeAllViews();
        textViews.clear();
        LayoutInflater inflater = LayoutInflater.from(getActivity());
        View tableRow = inflater.inflate(R.layout.matchdetails_result_nodraw, null);

        TextView homeNoDraw, awayNoDraw;
        homeNoDraw = tableRow.findViewById(R.id.result_nodraw_homeTeam);
        homeNoDraw.setText(homeNoDraw.getText() + " " + String.valueOf(getRandomFloatNumber()));

        awayNoDraw = tableRow.findViewById(R.id.result_nodraw_awayTeam);
        awayNoDraw.setText(awayNoDraw.getText() + " " + String.valueOf(getRandomFloatNumber()));

        textViews.add(homeNoDraw);
        textViews.add(awayNoDraw);

        homeNoDraw.setOnClickListener(click -> {
            this.betTypes = BetTypes.HOME_NO_DRAW;
            markSelectedView(homeNoDraw);
            extractFloatValue(homeNoDraw);
        });
        awayNoDraw.setOnClickListener(click -> {
            this.betTypes = BetTypes.AWAY_NO_DRAW;
            markSelectedView(awayNoDraw);
            extractFloatValue(awayNoDraw);
        });


        tableLayout.addView(tableRow);
    }

    //sázka na neprohru
    private void showDoubleChanceBetType(){
        tableLayout.removeAllViews();
        textViews.clear();
        LayoutInflater inflater = LayoutInflater.from(getActivity());
        View tableRow = inflater.inflate(R.layout.matchdetails_result_doublechance, null);


        TextView homeNoLose, noDraw, awayNoLose;
        homeNoLose = tableRow.findViewById(R.id.result_doublechance_homeTeam);
        homeNoLose.setText(homeNoLose.getText() + " " + String.valueOf(getRandomFloatNumber()));

        noDraw = tableRow.findViewById(R.id.result_doublechance_noDraw);
        noDraw.setText(noDraw.getText() + " " + String.valueOf(getRandomFloatNumber()));

        awayNoLose = tableRow.findViewById(R.id.result_doublechance_awayTeam);
        awayNoLose.setText(awayNoLose.getText() + " " + String.valueOf(getRandomFloatNumber()));


        textViews.add(homeNoLose);
        textViews.add(noDraw);
        textViews.add(awayNoLose);

        homeNoLose.setOnClickListener(click -> {
            this.betTypes = BetTypes.HOME_DOUBLE_CHANCE;
            markSelectedView(homeNoLose);
            extractFloatValue(homeNoLose);
        });
        noDraw.setOnClickListener(click -> {
            this.betTypes = BetTypes.NO_DRAW;
            markSelectedView(noDraw);
            extractFloatValue(noDraw);
        });
        awayNoLose.setOnClickListener(click -> {
            this.betTypes = BetTypes.AWAY_DOUBLE_CHANCE;
            markSelectedView(awayNoLose);
            extractFloatValue(awayNoLose);
        });
        tableLayout.addView(tableRow);
    }


    //naplnění hodnot do sázky
    private void realizeBet() throws Exception {
        Bet bet = new Bet();
        bet.setUserId(mainActivity.getUser().getId());

        int betAmount = Integer.valueOf(this.betValue.getText().toString());
        if(mainActivity.getUser().getBalance() < betAmount){
            Toast.makeText(getActivity(), "Nedostatečný zůstatek", Toast.LENGTH_SHORT).show();
            return;
        }else{
            bet.setValue(betAmount);
            bet.setEventId(eventId);
            bet.setOdd(selectedOdd);
            if(betTypes != null){
                bet.setBetType(betTypes.getValue());
            }
        }

        createBet(bet, betAmount);
    }

    //vytvoření sázky
    private void createBet(Bet bet, int betAmount){
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {

                    boolean result = BetRestClient.CREATE_BET(bet);
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            if(result){
                                mainActivity.updateBalance(-betAmount);
                                mainActivity.changeFragment(new ProfileFragmanet());
                            }else{
                                Toast.makeText(getActivity(), "Something went wrong", Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
                } catch (Exception e) {
                    System.out.println("ERROR ON CREATING BET");
                }
            }
        });
    }

}