package com.example.thomasbet;

import android.os.Bundle;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.Toast;

import com.example.thomasbet.RestClients.EventRestClient;
import com.example.thomasbet.customDataTypes.event.AdapterEventFinished;
import com.example.thomasbet.customDataTypes.event.ModelEvent;

import java.util.List;
import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class MatchResultsFragment extends Fragment implements AdapterView.OnItemSelectedListener {
    private Spinner matchesCategory;

    private MainActivity mainActivity;
    private RecyclerView events;
    private Button eventLoadMore_btn;
    private Button eventLoadless_btn;
    private AdapterEventFinished adapterEvent;
    private int currentPage;
    private int currentOffset = 2;
    private int pagesCount;

    private String sportType;

    public MatchResultsFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_match_results, container, false);

        matchesCategory = view.findViewById(R.id.match_results_sportType_spn);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(getContext(), R.array.spinner_SportCategories2, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        matchesCategory.setAdapter(adapter);

        matchesCategory.setOnItemSelectedListener(this);

        this.adapterEvent = new AdapterEventFinished(getContext(), (MainActivity) getActivity(), this);
        mainActivity = (MainActivity) getActivity();
        sportType = "all";

        events = view.findViewById(R.id.match_results_allmatches);
        eventLoadMore_btn = view.findViewById(R.id.match_results_loadmore_btn);
        eventLoadless_btn = view.findViewById(R.id.match_results_loadless_btn);

        eventLoadMore_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (currentPage < pagesCount - 1) {
                    currentPage++;
                    updateButtons();
                    loadEvents(currentOffset, currentPage, sportType);
                } else {
                    eventLoadMore_btn.setVisibility(View.GONE);
                }
            }
        });
        eventLoadless_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                currentPage--;
                updateButtons();
                loadEvents(currentOffset, currentPage, sportType);
                if (currentPage == 0) {
                    eventLoadless_btn.setVisibility(View.GONE);
                }
            }
        });

        if (currentPage == 0) {
            eventLoadless_btn.setVisibility(View.GONE);
        }

        if (currentPage == pagesCount - 1) {
            eventLoadMore_btn.setVisibility(View.GONE);
        }

        return view;
    }

    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        switch(parent.getItemAtPosition(position).toString()) {
            case "Všechno":
                this.sportType = "all";
                break;
            case "Fotbal":
                this.sportType = "football";
                break;
            case "Hokej":
                this.sportType = "ice_hockey";
                break;
        }

        loadEvents(currentOffset, currentPage, sportType);
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {
        //loadEvents(currentOffset, currentPage, "all");
    }

    private void loadEvents(int offset, int page, String sportType){
        this.adapterEvent.removeModels();
        System.out.println("REMOVING MODELS");

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    pagesCount = EventRestClient.GET_PAGES_COUNT(offset, sportType, "finished");
                    //List<ModelEvent> models = EventRestClient.GET_FINISHED_EVENTS_PAGED(offset, page, sportType);

                    EventRestClient.GET_FINISHED_EVENTS_PAGED(offset, page, sportType).forEach(model -> {
                        adapterEvent.addModelEvent(model);
                    });

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            if(adapterEvent.getEventArrayList().isEmpty()){
                                Toast.makeText(getActivity(), "Nebyly odehrány žádné zápasy", Toast.LENGTH_SHORT).show();
                            }
                            events.setAdapter(adapterEvent);
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e.getMessage());
                }
            }
        });
    }

    private void updateButtons(){
        if(currentPage == 0){
            eventLoadless_btn.setVisibility(View.GONE);
        }else{
            eventLoadless_btn.setVisibility(View.VISIBLE);
        }

        if(currentPage < pagesCount - 1){
            eventLoadMore_btn.setVisibility(View.VISIBLE);
        }else{
            eventLoadMore_btn.setVisibility(View.GONE);
        }
    }
}