package com.example.thomasbet;

import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import com.example.thomasbet.RestClients.BetRestClient;
import com.example.thomasbet.RestClients.EventRestClient;
import com.example.thomasbet.customDataTypes.betting.AdapterMyBets;
import com.example.thomasbet.customDataTypes.event.AdapterEventUpcoming;
import com.example.thomasbet.interfaces.UserUpdateListener;
import com.example.thomasbet.model.UserEntity;

import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class ProfileFragmanet extends Fragment implements UserUpdateListener {

    private static int DEPOSIT_MENU_VALUE = 0;
    private static int TICKET_HISTORY_MENU_VALUE = 1;
    private Button depositButton;
    private MainActivity mainActivity;
    private TextView balanceValue;
    private TextView profile_loggedUser;
    private TextView role_loggedUser;
    private Button myStrategies;
    private Button myArticles;



    private RecyclerView bets;
    private Button eventLoadMore_btn;
    private Button eventLoadless_btn;
    private AdapterMyBets adapterMyBets;
    private int currentPage;
    private int currentOffset = 2;
    private int pagesCount;



    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
    }

    public ProfileFragmanet() {

    }


    public static ProfileFragmanet newInstance(String param1, String param2) {
        ProfileFragmanet fragment = new ProfileFragmanet();

        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_profile, container, false);


        myStrategies = view.findViewById(R.id.profile_better_strategies);
        myArticles = view.findViewById(R.id.profile_better_articles);


        this.mainActivity = (MainActivity) getActivity();
        mainActivity.addListener(this);

        balanceValue = view.findViewById(R.id.balanceValue);
        balanceValue.setText(String.valueOf(mainActivity.getUser().getBalance()));

        profile_loggedUser = view.findViewById(R.id.profile_loggedUser);
        String nickname = mainActivity.getUser().getNickname();
        profile_loggedUser.setText(nickname);

        role_loggedUser = view.findViewById(R.id.profile_role_loggedUser);
        int role = mainActivity.getUser().getRole_id();
        if (role == 1){
            role_loggedUser.setText("Administrátor");
        } else if (role == 2) {
            role_loggedUser.setText("Profesionální sázkař");
        }else if (role == 3) {
            role_loggedUser.setText("Sázkař");
            myStrategies.setVisibility(View.GONE);
            myArticles.setVisibility(View.GONE);
        }

        Button DepositButton = view.findViewById(R.id.profile_deposit_button);
        DepositButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                MainActivity mainActivity = (MainActivity) getActivity();
                mainActivity.changeFragment(new DepositFragment());
            }
        });


        myStrategies.setOnClickListener(click -> {
            mainActivity.changeFragment(new StrategyFragment(0, Long.valueOf(mainActivity.getUser().getId())));
        });

        myArticles.setOnClickListener(click -> {
            mainActivity.changeFragment(new BlogFragment(0, Long.valueOf(mainActivity.getUser().getId())));
        });





        //Vsazené tikety
        this.adapterMyBets = new AdapterMyBets(getContext(), (MainActivity) getActivity(), this);

        bets = view.findViewById(R.id.my_bets);
        eventLoadMore_btn = view.findViewById(R.id.my_tickets_loadmore);
        eventLoadless_btn = view.findViewById(R.id.my_tickets_loadless);

        loadBets(currentOffset, currentPage);
        eventLoadMore_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(currentPage < pagesCount -1){
                    currentPage++;
                    updateButtons();
                    loadBets(currentOffset, currentPage);
                }else{
                    eventLoadMore_btn.setVisibility(View.GONE);
                }
            }
        });
        eventLoadless_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                currentPage--;
                updateButtons();
                loadBets(currentOffset, currentPage);
                if (currentPage == 0) {
                    eventLoadless_btn.setVisibility(View.GONE);
                }
            }
        });

        if (currentPage == 0) {
            eventLoadless_btn.setVisibility(View.GONE);
        }

        if(currentPage == pagesCount-1){
            eventLoadMore_btn.setVisibility(View.GONE);
        }
        return view;
    }

    @Override
    public void onUserUpdate(UserEntity user) {
        if (user != null) {
            this.balanceValue.setText(String.valueOf(user.getBalance()));
        } else {
            this.balanceValue.setVisibility(View.INVISIBLE);
        }

    }


    private void loadBets(int offset, int page){
        this.adapterMyBets.removeModels();

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    pagesCount = BetRestClient.GET_PAGES_COUNT(offset, (long) mainActivity.getUser().getId());
                    BetRestClient.GET_MY_BETS(offset, page, (long) mainActivity.getUser().getId()).forEach(bet -> {
                        adapterMyBets.addModelBet(bet);
                    });

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            bets.setAdapter(adapterMyBets);
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });
    }



    private void updateButtons(){
        if(currentPage == 0){
            eventLoadless_btn.setVisibility(View.GONE);
        }else{
            eventLoadless_btn.setVisibility(View.VISIBLE);
        }

        if(currentPage < pagesCount - 1){
            eventLoadMore_btn.setVisibility(View.VISIBLE);
        }else{
            eventLoadMore_btn.setVisibility(View.GONE);
        }
    }

}