package com.example.thomasbet;

import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputBinding;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.model.UserEntity;

import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import at.favre.lib.crypto.bcrypt.BCrypt;


public class RegisterFragment extends Fragment {
    private EditText emailField;
    private EditText passwordField;
    private EditText confirmPasswordField;
    private EditText nicknameField;

    private OnRegisterSuccessListener onRegisterSuccessListener;
    private OnLoginClickListener onLoginClickListener;

    public interface OnRegisterSuccessListener {
        void onRegisterSuccess();
    }

    public interface OnLoginClickListener {
        void moveToLoginFragment();
    }

    public RegisterFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_register, container, false);

        emailField = view.findViewById(R.id.email_register);
        nicknameField = view.findViewById(R.id.nickname);
        passwordField = view.findViewById(R.id.password_register);
        confirmPasswordField = view.findViewById(R.id.password_confirm_register);
        Button registerButton = view.findViewById(R.id.register_btn);
        TextView loginText = view.findViewById(R.id.login_redirect_text);

        emailField.setOnFocusChangeListener((v, hasFocus) ->{
           String text = emailField.getText().toString();
            if (hasFocus && text.contains("@")) {
                // Set drawableEnd when EditText gains focus and contains "@"
                emailField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.baseline_done_24, 0);
            } else if (text.contains("@") && !hasFocus){
                // Set drawableEnd when EditText loses focus
                emailField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.baseline_done_24, 0);
            } else {
                // Set drawableEnd when EditText loses focus
                emailField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.ic_cross, 0);
            }
        });

        nicknameField.setOnFocusChangeListener((vv, hasFocus) ->{
            String text2 = emailField.getText().toString();
            if (!hasFocus && !text2.isEmpty()) {
                nicknameField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.baseline_done_24, 0);
            } else {
                // Set drawableEnd when EditText loses focus
                nicknameField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.ic_cross, 0);
            }
        });


        confirmPasswordField.setOnFocusChangeListener((v, hasFocus) -> {
            validateFields(hasFocus);
        });

        loginText.setOnClickListener((text) -> {
            onLoginClickListener.moveToLoginFragment();
        });


        registerButton.setOnClickListener((button) -> {
            if(validateFields(false)){
                registerUser(emailField.getText().toString(),
                        passwordField.getText().toString(),
                        confirmPasswordField.getText().toString(),
                        nicknameField.getText().toString());
            }
        });
        return view;
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);

        onLoginClickListener = (RegisterFragment.OnLoginClickListener) context;
        onRegisterSuccessListener = (RegisterFragment.OnRegisterSuccessListener) context;

    }

    //registrace uživatele
    public void registerUser(String email, String password, String confirmPassword, String nickname){

        //podle zadaných hodnot nastaví uživatele, prvotní role je pevně 3 (uživatel) - musí změnit ručně administrátor
        UserEntity newUser = new UserEntity(email, 3, nickname, 0, hashPassword(password));

        //probíhá v pozadí, tudíž nebude zatěžovat aplikaci a běžet na hlavním vlákně
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                String result;
                try {
                    //porovnávání API callu s DB
                    result = UserRestClient.REGISTER_USER(newUser);
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            if(result.equals("Registrace byla úspěšná")){
                                Toast.makeText(getActivity(), result, Toast.LENGTH_SHORT).show();
                                onRegisterSuccessListener.onRegisterSuccess();
                            }else{
                                Toast.makeText(getActivity(), result, Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });
    }

    // ověření, že hesla jsou shodná, pokud ne, vyskočí chybová hláška
    private boolean validateFields(boolean hasFocus){
        if (!hasFocus) {
            String password = passwordField.getText().toString();
            String confirmPassword = confirmPasswordField.getText().toString();

            // pokud se pole hesla neshoduje s polem potvrzení hesla
            if (!password.equals(confirmPassword)) {
                confirmPasswordField.setError("Hesla se neshodují");
                return false;
            }else {
                // pokud se pole hesla shoduje s polem potvrzení hesla, nastaví se na konec pole ikona
                confirmPasswordField.setError(null);
                passwordField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.baseline_done_24, 0);
                confirmPasswordField.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.baseline_done_24, 0);
                return true;
            }
        }
        return true;
    }

    // zašifrování hesla pomocí knihovny Bcrypt o síle 10
    public String hashPassword(String password) {
        return BCrypt.withDefaults().hashToString(10, password.toCharArray());
    }
}