package com.example.thomasbet.RestClients;

import com.example.thomasbet.customDataTypes.blog.ModelBlog;
import com.example.thomasbet.customDataTypes.strategy.ModelStrategy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class BlogRestClient {

    //získání všech blogů bez stránkování - využito ve fázi testování funkčnosti
    public static List<ModelBlog> GET_ALL_BLOGS() throws IOException, JSONException {
        URL url = new URL("http://10.0.2.2:8080/api/blogs");

        ArrayList<ModelBlog> blogArrayList = new ArrayList<>();

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();

            JSONArray jsonArray = new JSONArray(jsonResponse.toString());

            Gson gson = new GsonBuilder()
                    .serializeNulls()
                    .create();

            for (int i=0; i<jsonArray.length(); i++) {
                JSONObject blogObject = jsonArray.getJSONObject(i);
                ModelBlog model = gson.fromJson(blogObject.toString(), ModelBlog.class);
                blogArrayList.add(model);
            }

            if(!blogArrayList.isEmpty()){
                return blogArrayList;
            }else{
                return null;
            }
        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }

    //získání všech blogů se stránkováním
    public static List<ModelBlog> GET_BLOGS_PAGED(int offset, int page) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/blogs/";
        url = url.concat(String.valueOf(offset) + "/");
        url = url.concat(String.valueOf(page));
        URL apiUrl = new URL(url);

        ArrayList<ModelBlog> blogArrayList = new ArrayList<>();

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();

            JSONObject jsonResponseObject = new JSONObject(jsonResponse.toString());
            JSONObject responseKey = new JSONObject(jsonResponseObject.get("response").toString());
            JSONArray jsonArray = new JSONArray(responseKey.get("content").toString());

            Gson gson = new GsonBuilder()
                    .serializeNulls()
                    .create();

            for (int i=0; i<jsonArray.length(); i++) {
                JSONObject blogObject = jsonArray.getJSONObject(i);
                ModelBlog model = gson.fromJson(blogObject.toString(), ModelBlog.class);
                blogArrayList.add(model);
            }

            if(!blogArrayList.isEmpty()){
                return blogArrayList;
            }else{
                return null;
            }
        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }


    //žískání všech blogů uživatele
    public static List<ModelBlog> GET_BLOGS_BY_USER_ID(Long userId, int page, int offset) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/blogs?userId=";
        url = url.concat(String.valueOf(userId) + "&");
        url = url.concat("offset=" + String.valueOf(offset) + "&");
        url = url.concat("page=" + String.valueOf(page));
        URL apiUrl = new URL(url);

        System.out.println(apiUrl);

        ArrayList<ModelBlog> blogArrayList = new ArrayList<>();

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();

            JSONObject jsonResponseObject = new JSONObject(jsonResponse.toString());
            JSONObject responseKey = new JSONObject(jsonResponseObject.get("response").toString());
            JSONArray jsonArray = new JSONArray(responseKey.get("content").toString());

            Gson gson = new GsonBuilder()
                    .serializeNulls()
                    .create();

            for (int i=0; i<jsonArray.length(); i++) {
                JSONObject modelObject = jsonArray.getJSONObject(i);
                ModelBlog model = gson.fromJson(modelObject.toString(), ModelBlog.class);
                blogArrayList.add(model);
            }

            if(!blogArrayList.isEmpty()){
                return blogArrayList;
            }else{
                return null;
            }
        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }


    //získání počtu vrácených stránek pro správnost stránkování
    public static int GET_PAGES_COUNT(int offset, Long userId, boolean myArticles) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/blogs/count?offset=";
        url = url.concat(String.valueOf(offset) + "&");
        String myArticlesParam;
        if(myArticles){
            myArticlesParam = "true";
        }else{
            myArticlesParam = "false";
        }
        url = url.concat("myArticles=" + myArticlesParam + "&");
        url = url.concat("userId=" + String.valueOf(userId));

        URL apiUrl = new URL(url);

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();


            return Integer.parseInt(jsonResponse.toString());
        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }


    //smazání blogu
    public static void DELETE_BLOG(Long blogId) throws IOException, JSONException {
        URL url = new URL("http://10.0.2.2:8080/api/blogs/" + String.valueOf(blogId));
        try{
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("DELETE");

            int responseCode = urlConnection.getResponseCode();
            System.out.println("Response Code: " + responseCode);


            urlConnection.disconnect();

        } catch (Exception e){
            throw e;
        }
    }


    //úprava blogu
    public static boolean UPDATE_BLOG(ModelBlog blog) throws Exception {
        URL url = new URL("http://10.0.2.2:8080/api/blogs");

        Gson gson = new Gson();
        String json = gson.toJson(blog);
        System.out.println(json);

        try {
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("PATCH");
            urlConnection.setRequestProperty("Content-Type", "application/json");
            urlConnection.setDoOutput(true);
            urlConnection.connect();

            OutputStream outputStream = urlConnection.getOutputStream();
            outputStream.write(json.getBytes());
            outputStream.flush();
            outputStream.close();

            StringBuilder jsonResponse = new StringBuilder();

            int responseCode = urlConnection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                /*
                InputStream inputStream = urlConnection.getInputStream();
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    jsonResponse.append(line);
                }
                */
                return true;
            }else{
                return false;
            }
        }catch(Exception E){
            return false;
        }
    }


    //vytvoření blogu
    public static boolean CREATE_BLOG(ModelBlog blog) throws Exception {
        URL url = new URL("http://10.0.2.2:8080/api/blogs");

        Gson gson = new Gson();
        String json = gson.toJson(blog);
        System.out.println(json);


        try {
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("POST");
            urlConnection.setRequestProperty("Content-Type", "application/json");
            urlConnection.setDoOutput(true);
            urlConnection.connect();

            OutputStream outputStream = urlConnection.getOutputStream();
            outputStream.write(json.getBytes());
            outputStream.flush();
            outputStream.close();

            StringBuilder jsonResponse = new StringBuilder();

            int responseCode = urlConnection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                /*
                InputStream inputStream = urlConnection.getInputStream();
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    jsonResponse.append(line);
                }
                */
                return true;
            }else{
                return false;
            }
        }catch(Exception E){
            return false;
        }
    }
}
