package com.example.thomasbet.RestClients;

import com.example.thomasbet.customDataTypes.event.ModelEvent;
import com.example.thomasbet.customDataTypes.strategy.ModelStrategy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class EventRestClient {

    //získání zápasů do rozvrhu zápasů, pouze těch které nejsou odehrány, přidána možnost filtrace podle typu sportu
    public static List<ModelEvent> GET_UPCOMING_EVENTS_PAGED(int offset, int page, String sportType) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/events/";
        url = url.concat(String.valueOf(offset) + "/");
        url = url.concat(String.valueOf(page) + "/");
        url = url.concat(sportType + "/");
        url = url.concat("upcoming");
        URL apiUrl = new URL(url);

        System.out.println("URL: " + apiUrl);

        ArrayList<ModelEvent> eventArrayList = new ArrayList<>();

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();

            JSONObject jsonResponseObject = new JSONObject(jsonResponse.toString());
            JSONObject responseKey = new JSONObject(jsonResponseObject.get("response").toString());
            JSONArray jsonArray = new JSONArray(responseKey.get("content").toString());

            Gson gson = new GsonBuilder()
                    .serializeNulls()
                    .create();

            for (int i=0; i<jsonArray.length(); i++) {
                JSONObject eventObject = jsonArray.getJSONObject(i);
                ModelEvent model = gson.fromJson(eventObject.toString(), ModelEvent.class);
                eventArrayList.add(model);
            }

            return eventArrayList;

        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }

    //získání zápasů do výsledků zápasů, pouze těch které jsou odehrány, přidána možnost filtrace podle typu sportu
    public static List<ModelEvent> GET_FINISHED_EVENTS_PAGED(int offset, int page, String sportType) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/events/";
        url = url.concat(String.valueOf(offset) + "/");
        url = url.concat(String.valueOf(page) + "/");
        url = url.concat(sportType + "/");
        url = url.concat("finished");
        URL apiUrl = new URL(url);

        System.out.println("URL: " + apiUrl);

        ArrayList<ModelEvent> eventArrayList = new ArrayList<>();

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();

            JSONObject jsonResponseObject = new JSONObject(jsonResponse.toString());
            JSONObject responseKey = new JSONObject(jsonResponseObject.get("response").toString());
            JSONArray jsonArray = new JSONArray(responseKey.get("content").toString());

            Gson gson = new GsonBuilder()
                    .serializeNulls()
                    .create();

            for (int i=0; i<jsonArray.length(); i++) {
                JSONObject eventObject = jsonArray.getJSONObject(i);
                ModelEvent model = gson.fromJson(eventObject.toString(), ModelEvent.class);
                eventArrayList.add(model);
            }

            return eventArrayList;
        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }


    //počet vrácených dat, pro správnost stránkování
    public static int GET_PAGES_COUNT(int offset, String sportType, String state) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/events/count?offset=";
        url = url.concat(String.valueOf(offset) + "&");
        url = url.concat("sportType=" + sportType);
        url = url.concat("&state=" + state);
        URL apiUrl = new URL(url);

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();


            return Integer.parseInt(jsonResponse.toString());
        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }


    //počet vrácených zápasů podle ID
    public static ModelEvent GET_EVENT_BY_ID(String id) throws IOException, JSONException {
        String url = "http://10.0.2.2:8080/api/events/" + id;
        URL apiUrl = new URL(url);


        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) apiUrl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.setRequestProperty("Accept", "application/json");
            //urlConnection.connect();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            urlConnection.disconnect();

            JSONObject response = new JSONObject(jsonResponse.toString());

            Gson gson = new GsonBuilder()
                    .serializeNulls()
                    .create();

            ModelEvent model = gson.fromJson(response.toString(), ModelEvent.class);

            return model;

        } catch (Exception e){
            e.printStackTrace();
            throw e;
        }
    }
}
