package com.example.thomasbet.RestClients;

import android.net.Uri;

import com.example.thomasbet.model.UserEntity;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;

public class UserRestClient {

    //využito při filtraci uživatelů v Admin Panelu
    public static List<UserEntity> GET_USERS_BY_EMAIL_OR_NICKNAME(String email, String nickname) throws IOException {
        String baseUrl = "http://10.0.2.2:8080/api";
        String endpoint = "/users";
        Uri.Builder builder = Uri.parse(baseUrl + endpoint).buildUpon();
        if(email != null) builder.appendQueryParameter("email", email);
        if(nickname != null) builder.appendQueryParameter("nickname", nickname);

        String urlString = builder.build().toString();
        URL url = new URL(urlString);

        StringBuilder jsonResponse = new StringBuilder();
        try{
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.connect();

            InputStream inputStream = urlConnection.getInputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
            bufferedReader.close();

            Gson gson = new Gson();
            Type listType = new TypeToken<List<UserEntity>>(){}.getType();
            List<UserEntity> responseObjectList = gson.fromJson(jsonResponse.toString(), listType);

            for (UserEntity responseObject : responseObjectList) {
                System.out.println("RESPONSE USER ENTITY OBJECT : " + responseObject.toString());
            }

            return responseObjectList;

        }catch(Exception E){
            System.out.println("EXCEPTION1111111: " + E);
            return null;
        }
    }


    //vrácení uživatele podle jeho ID
    public static UserEntity GET_USER_BY_ID(int id) throws IOException {
        String baseUrl = "http://10.0.2.2:8080/api";
        String endpoint = "/users/";
        Uri.Builder builder = Uri.parse(baseUrl + endpoint + String.valueOf(id)).buildUpon();

        String urlString = builder.build().toString();
        URL url = new URL(urlString);

        StringBuilder jsonResponse = new StringBuilder();
        try{
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.connect();



            if(urlConnection.getResponseCode() == 200){
                InputStream inputStream = urlConnection.getInputStream();
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));

                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    jsonResponse.append(line);
                }
                bufferedReader.close();

                Gson gson = new Gson();
                return gson.fromJson(jsonResponse.toString(), UserEntity.class);
            }else if(urlConnection.getResponseCode() == 500){
                System.out.println("IN 500 CODE");
                return null;
            }else{
                System.out.println("NOT 200 or 500: " + String.valueOf(urlConnection.getResponseCode()));
                return null;
            }
        }catch(Exception E){
            System.out.println("Error in api fetch: " + E.getMessage());
            return null;
        }
    }


    //změna na kontě uživatele
    public static UserEntity UPDATE_USER_BALANCE(UserEntity user, int change) throws Exception {
        URL url = new URL("http://10.0.2.2:8080/api/users?deposit=" + String.valueOf(change));

        Gson gson = new Gson();
        String json = gson.toJson(user);

        try {
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("PATCH");
            urlConnection.setRequestProperty("Content-Type", "application/json");
            urlConnection.setDoOutput(true);
            urlConnection.connect();

            OutputStream outputStream = urlConnection.getOutputStream();
            outputStream.write(json.getBytes());
            outputStream.flush();
            outputStream.close();

            StringBuilder jsonResponse = new StringBuilder();

            int responseCode = urlConnection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {

                InputStream inputStream = urlConnection.getInputStream();
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    jsonResponse.append(line);
                }

                return gson.fromJson(jsonResponse.toString(), UserEntity.class);
            } else {
                return user;
            }
        }catch(Exception E){
            throw new Exception();
        }
    }


    //změna uživatele - role
    public static boolean UPDATE_USER(UserEntity user) throws Exception {
        URL url = new URL("http://10.0.2.2:8080/api/users");

        Gson gson = new Gson();
        String json = gson.toJson(user);

        try {
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("PATCH");
            urlConnection.setRequestProperty("Content-Type", "application/json");
            urlConnection.setDoOutput(true);
            urlConnection.connect();

            OutputStream outputStream = urlConnection.getOutputStream();
            outputStream.write(json.getBytes());
            outputStream.flush();
            outputStream.close();

            StringBuilder jsonResponse = new StringBuilder();

            int responseCode = urlConnection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {

                InputStream inputStream = urlConnection.getInputStream();
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    jsonResponse.append(line);
                }
                return true;
            }else{
                return false;
            }
        }catch(Exception E){
            return false;
        }
    }


    //vrácení uživatele po přihlášení
    public static UserEntity GET_USER_AFTER_LOGIN(String email) throws IOException {
        URL url = new URL("http://10.0.2.2:8080/api/users" + "/login?email=" + email);

        StringBuilder jsonResponse = new StringBuilder();

        try{
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.connect();

            int responseCode = urlConnection.getResponseCode();

            InputStream inputStream = urlConnection.getInputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));

            String line;
            while ((line = bufferedReader.readLine()) != null) {
                jsonResponse.append(line);
            }
        }catch(Exception E){
            return null;
        }


        Gson gson = new Gson();
        UserEntity userEntity = gson.fromJson(jsonResponse.toString(), UserEntity.class);

        return userEntity;
    }

    //registrace uživatele
    public static String REGISTER_USER(UserEntity newUser) throws IOException {
        //nastavení URL adresy (v této fázi adresa uzpůsobena pro emulované telefony) TODO: Změnit na fyzické zařízení
        URL url = new URL("http://10.0.2.2:8080/api/users" + "/register");

        //konverze dat
        Gson gson = new Gson();
        String json = gson.toJson(newUser);

        try {
            //připojení k DB formou POST
            HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
            urlConnection.setRequestMethod("POST");
            urlConnection.setRequestProperty("Content-Type", "application/json");
            urlConnection.setDoOutput(true);
            urlConnection.connect();

            OutputStream outputStream = urlConnection.getOutputStream();
            outputStream.write(json.getBytes());
            outputStream.flush();
            outputStream.close();

            //ošetření chybových hlášek
            int responseCode = urlConnection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                return "Registrace byla úspěšná";
            } else {
                return "Tento email byl již použit";
            }
        }catch(Exception E){
            return "Něco se pokazilo, zkuste to prosím později";
        }
    }

}




