package com.example.thomasbet;

import android.os.Bundle;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;

import com.example.thomasbet.RestClients.StrategyRestClient;
import com.example.thomasbet.customDataTypes.strategy.AdapterStrategy;

import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class StrategyFragment extends Fragment {

    // pro blog
    private RecyclerView strategyPosts;
    private Button strategyLoadMore_btn;
    private Button strategyLoadless_btn;
    private AdapterStrategy adapterPost;
    private int currentPage;
    private int currentOffset = 2;
    private int pagesCount;
    private Long userIdFilter = 0L;



    public StrategyFragment() {
        this.currentPage = 0;
    }

    public StrategyFragment(int page, Long userIdFilter){
        this.currentPage = page;
        this.userIdFilter = userIdFilter;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_strategy, container, false);
        this.adapterPost = new AdapterStrategy(getContext(), (MainActivity) getActivity(), this);
        strategyPosts = view.findViewById(R.id.strategy_posts);
        strategyLoadMore_btn = view.findViewById(R.id.strategy_loadmore_btn);
        strategyLoadless_btn = view.findViewById(R.id.strategy_loadless_btn);


        loadposts(currentOffset, currentPage);
        strategyLoadMore_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(currentPage < pagesCount -1){
                    currentPage++;
                    updateButtons();
                    loadposts(currentOffset, currentPage);
                }else{
                    strategyLoadMore_btn.setVisibility(View.GONE);
                }
            }
        });
        strategyLoadless_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                currentPage--;
                updateButtons();
                loadposts(currentOffset, currentPage);
                if (currentPage == 0) {
                    strategyLoadless_btn.setVisibility(View.GONE);
                }
            }
        });

        if (currentPage == 0) {
            strategyLoadless_btn.setVisibility(View.GONE);
        }

        if(currentPage == pagesCount-1){
            strategyLoadMore_btn.setVisibility(View.GONE);
        }
        return view;
    }

    private void loadposts(int offset, int page){
        this.adapterPost.removeModels();

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {

                    if(userIdFilter == 0){
                        pagesCount = StrategyRestClient.GET_PAGES_COUNT(offset, 0L, false);
                        Objects.requireNonNull(StrategyRestClient.GET_STRATEGIES_PAGED(offset, page)).forEach(model -> {
                            adapterPost.addModelPost(model);
                        });
                    }else{
                        pagesCount = StrategyRestClient.GET_PAGES_COUNT(offset, userIdFilter, true);
                        Objects.requireNonNull(StrategyRestClient.GET_STRATEGIES_BY_USER_ID(userIdFilter, page, offset)).forEach(model -> {
                            adapterPost.addModelPost(model);
                        });
                    }

                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            strategyPosts.setAdapter(adapterPost);
                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });
    }

    private void updateButtons(){
        if(currentPage == 0){
            strategyLoadless_btn.setVisibility(View.GONE);
        }else{
            strategyLoadless_btn.setVisibility(View.VISIBLE);
        }

        if(currentPage < pagesCount - 1){
            strategyLoadMore_btn.setVisibility(View.VISIBLE);
        }else{
            strategyLoadMore_btn.setVisibility(View.GONE);
        }
    }

    public void deleteStrategy(Long id) {
        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    StrategyRestClient.DELETE_STRATEGY(id);
                    handler.post(new Runnable() {
                        @Override
                        public void run() {

                        }
                    });
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        });

        loadposts(currentOffset, currentPage);
    }


    public void refresh(){
        loadposts(currentOffset, currentPage);
    }
}