package com.example.thomasbet.customDataTypes.betting;

import android.content.Context;
import android.media.metrics.Event;
import android.os.Handler;
import android.os.Looper;
import android.text.InputType;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.thomasbet.BlogFragment;
import com.example.thomasbet.Enums.BetTypes;
import com.example.thomasbet.MainActivity;
import com.example.thomasbet.MatchResultsFragment;
import com.example.thomasbet.ProfileFragmanet;
import com.example.thomasbet.R;
import com.example.thomasbet.RestClients.BlogRestClient;
import com.example.thomasbet.RestClients.EventRestClient;
import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.customDataTypes.betting.AdapterMyBets;
import com.example.thomasbet.customDataTypes.blog.ModelBlog;
import com.example.thomasbet.customDataTypes.event.ModelEvent;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class AdapterMyBets extends RecyclerView.Adapter<AdapterMyBets.HolderBets>{

    private Context context;
    private MainActivity mainActivity;
    private ProfileFragmanet currentFragment;
    private ArrayList<Bet> betArrayList = new ArrayList<>();

    public AdapterMyBets(Context context, MainActivity mainActivity, ProfileFragmanet profileFragmanet) {
        this.context = context;
        this.mainActivity = mainActivity;
        this.currentFragment = profileFragmanet;
    }

    @NonNull
    @Override
    public AdapterMyBets.HolderBets onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.ticketbody, parent, false);
        return new AdapterMyBets.HolderBets(view);
    }

    public void addModelBet(Bet bet){
        this.betArrayList.add(bet);
    }

    public void removeModels(){
        betArrayList.clear();
    }

    //vázání dat na konkrétní pozice v RecyclerView
    @Override
    public void onBindViewHolder(@NonNull AdapterMyBets.HolderBets holder, int position) {
        Bet model = betArrayList.get(position);

        holder.result.setText("Nevyhodocneno");
        holder.possibleWin.setText(String.valueOf(model.getValue()*model.getOdd()));
        holder.betValue.setText(String.valueOf(model.getValue()));
        holder.betType.setText(BetTypes.GET_STRING(model.getBetType()));

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                ModelEvent modelEvent;
                try {
                    modelEvent = EventRestClient.GET_EVENT_BY_ID(model.getEventId());
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            if(modelEvent != null){
                                SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
                                holder.commenceTime.setText(sdf.format(new Date(modelEvent.getCommenceTime())));
                                holder.matchTitle.setText(modelEvent.getHomeTeam() + " vs " + modelEvent.getAwayTeam());
                            }else{
                                Toast.makeText(mainActivity, "Zápas se nenašel", Toast.LENGTH_SHORT).show();
                            }

                        }
                    });
                } catch (Exception e) {
                    System.out.println("ERROR ON FETCHING EVENT");
                }
            }
        });
    }



    @Override
    public int getItemCount() {
        return betArrayList.size();
    }

    public ArrayList<Bet> getBetArrayList() {
        return betArrayList;
    }

    static class HolderBets extends RecyclerView.ViewHolder {
        TextView result, commenceTime, matchTitle, betType, betValue, possibleWin;

        LinearLayout buttonsLinearLayout;

        //definování jednoho Holdera daty
        public HolderBets(@NonNull View itemView) {
            super(itemView);
            result = itemView.findViewById(R.id.ticket_result);
            commenceTime = itemView.findViewById(R.id.ticket_bet_time);
            matchTitle = itemView.findViewById(R.id.ticket_title);
            betType = itemView.findViewById(R.id.ticket_betType);
            betValue = itemView.findViewById(R.id.ticket_betAmount);
            possibleWin = itemView.findViewById(R.id.ticket_possibleWin);
        }
    }
}
