package com.example.thomasbet.customDataTypes.blog;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.InputType;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.thomasbet.BlogFragment;
import com.example.thomasbet.MainActivity;
import com.example.thomasbet.R;
import com.example.thomasbet.RestClients.BlogRestClient;
import com.example.thomasbet.RestClients.StrategyRestClient;
import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.StrategyFragment;
import com.example.thomasbet.customDataTypes.strategy.AdapterStrategy;
import com.example.thomasbet.customDataTypes.strategy.ModelStrategy;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

public class AdapterBlog extends RecyclerView.Adapter<AdapterBlog.HolderBlog> {
    private Context context;
    private MainActivity mainActivity;
    private BlogFragment currentFragment;
    private ArrayList<ModelBlog> blogArrayList = new ArrayList<>();

    public AdapterBlog(Context context, MainActivity mainActivity, BlogFragment blogFragment) {
        this.context = context;
        this.mainActivity = mainActivity;
        this.currentFragment = blogFragment;
    }

    @NonNull
    @Override
    public AdapterBlog.HolderBlog onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.blogbody, parent, false);
        return new AdapterBlog.HolderBlog(view);
    }

    public void addModelBlog(ModelBlog modelBlog){
        this.blogArrayList.add(modelBlog);
    }

    public void removeModels(){
        blogArrayList.clear();
    }

    //vázání dat na konkrétní pozice v RecyclerView
    @Override
    public void onBindViewHolder(@NonNull AdapterBlog.HolderBlog holder, int position) {
        ModelBlog model = blogArrayList.get(position);


        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    model.setUser(Objects.requireNonNull(UserRestClient.GET_USER_BY_ID(model.getUserId())));
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");

                            holder.blogPublisher.setText(model.getUser().getNickname());
                            holder.createdIn.setText(sdf.format(new Date(model.getDateNew())));
                            holder.editedAt.setText(sdf.format(new Date(model.getDateEdit())));
                            if(mainActivity.getUser().getRole_id() == 1 || model.getUserId() == mainActivity.getUser().getId()){
                                holder.buttonsLinearLayout.setVisibility(View.VISIBLE);
                                holder.blogEdit.setOnClickListener(click -> {
                                    holder.setEditMode();
                                });
                                holder.blogSave.setOnClickListener(click -> {
                                    holder.sendUpdateRequest(holder.mapHolderBlogToModelBlog(holder), mainActivity, currentFragment);
                                    holder.setViewMode(mainActivity);

                                });
                                holder.blogDelete.setOnClickListener(click -> {
                                    currentFragment.deleteBlog(model.getBlogId());
                                });
                            }else{
                                holder.buttonsLinearLayout.setVisibility(View.GONE);
                            }
                        }
                    });
                } catch (Exception e) {
                    System.out.println("ERROR ON USER GETNICKNAME");
                }
            }
        });


        holder.title.setText(model.getTitle());
        holder.blogBody.setText(model.getBody());
        holder.dateEdit = model.getDateEdit();
        holder.dateNew = model.getDateNew();
        holder.blogId = model.getBlogId();
        holder.userId = model.getUserId();

    }



    @Override
    public int getItemCount() {
        return blogArrayList.size();
    }

    public ArrayList<ModelBlog> getBlogArrayList() {
        return blogArrayList;
    }

    //definování třídy jednoho Holdera daty
    static class HolderBlog extends RecyclerView.ViewHolder{
        Button moreBtn;
        EditText title;
        Long dateEdit;
        Long dateNew;
        Integer userId;
        Long blogId;
        EditText blogBody,blogPublisher, createdIn, editedAt;
        Button blogEdit;
        Button blogDelete;
        Button blogSave;

        LinearLayout buttonsLinearLayout;

        //vázání dat s XML
        public HolderBlog(@NonNull View itemView) {
            super(itemView);
            moreBtn = itemView.findViewById(R.id.blog_loadmore_btn);
            title = itemView.findViewById(R.id.blog_title);
            //strategyImage = itemView.findViewById(R.id.strategy_image);
            blogBody = itemView.findViewById(R.id.blog_body);
            blogPublisher = itemView.findViewById(R.id.blog_publisher);
            blogPublisher.setInputType(InputType.TYPE_NULL);
            blogEdit = itemView.findViewById(R.id.blog_article_edit);
            blogDelete = itemView.findViewById(R.id.blog_article_delete);
            buttonsLinearLayout = itemView.findViewById(R.id.blog_articleControl_buttons);
            createdIn = itemView.findViewById(R.id.blog_createdIn);
            createdIn.setInputType(InputType.TYPE_NULL);
            editedAt = itemView.findViewById(R.id.blog_editedIn);
            editedAt.setInputType(InputType.TYPE_NULL);
            buttonsLinearLayout.setVisibility(View.GONE);

            blogSave = itemView.findViewById(R.id.blog_article_save);
            blogSave.setVisibility(View.GONE);
        }

        //pohled sázkaře
        private void setEditMode(){
            this.blogEdit.setVisibility(View.GONE);
            this.blogSave.setVisibility(View.VISIBLE);


            title.setInputType(InputType.TYPE_CLASS_TEXT);
            blogBody.setInputType(InputType.TYPE_CLASS_TEXT);
            blogPublisher.setInputType(InputType.TYPE_NULL);
        }

        //pohled uživatele
        private void setViewMode(MainActivity mainActivity){

            this.blogSave.setVisibility(View.GONE);
            this.blogEdit.setVisibility(View.VISIBLE);

            title.setInputType(InputType.TYPE_NULL);
            blogBody.setInputType(InputType.TYPE_NULL);
        }


        //přemapování dat
        private ModelBlog mapHolderBlogToModelBlog(AdapterBlog.HolderBlog holderBlog){
            ModelBlog modelBlog = new ModelBlog();
            modelBlog.setUserId(holderBlog.userId);
            modelBlog.setBlogId(holderBlog.blogId);
            modelBlog.setBody(holderBlog.blogBody.getText().toString());
            modelBlog.setTitle(holderBlog.title.getText().toString());
            modelBlog.setDateNew(dateNew);
            modelBlog.setDateEdit(System.currentTimeMillis());

            return modelBlog;
        }


        //Úprava blogu
        private void sendUpdateRequest(ModelBlog modelBlog, MainActivity mainActivity, BlogFragment blogFragment){
            Executor executor = Executors.newSingleThreadExecutor();
            Handler handler = new Handler(Looper.getMainLooper());

            executor.execute(new Runnable() {
                @Override
                public void run() {
                    try {
                        boolean result = BlogRestClient.UPDATE_BLOG(modelBlog);
                        handler.post(new Runnable() {
                            @Override
                            public void run() {
                                if(result){
                                    Toast.makeText(mainActivity, "Blog has been updated", Toast.LENGTH_SHORT).show();
                                    blogFragment.refresh();
                                }else{
                                    Toast.makeText(mainActivity, "Something went wrong", Toast.LENGTH_SHORT).show();
                                }

                            }
                        });
                    } catch (Exception e) {
                        System.out.println(e);
                    }
                }
            });
        }

    }

}
