package com.example.thomasbet.customDataTypes.event;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.thomasbet.MainActivity;
import com.example.thomasbet.MatchResultsFragment;
import com.example.thomasbet.R;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public class AdapterEventFinished extends RecyclerView.Adapter<AdapterEventFinished.HolderEventFinished>{
    private Context context;
    private MainActivity mainActivity;
    private MatchResultsFragment currentFragment;
    private ArrayList<ModelEvent> eventArrayList = new ArrayList<>();

    public AdapterEventFinished(Context context, MainActivity mainActivity, MatchResultsFragment eventFragment) {
        this.context = context;
        this.mainActivity = mainActivity;
        this.currentFragment = eventFragment;
    }

    @NonNull
    @Override
    public AdapterEventFinished.HolderEventFinished onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.match_result_body, parent, false);
        return new AdapterEventFinished.HolderEventFinished(view);
    }

    public void addModelEvent(ModelEvent modelEvent){
        this.eventArrayList.add(modelEvent);
    }

    public void removeModels(){
        eventArrayList.clear();
    }




    //Struktura výsledků zápasu v Holderu
    @Override
    public void onBindViewHolder(@NonNull AdapterEventFinished.HolderEventFinished holder, int position) {
        ModelEvent model = eventArrayList.get(position);

        holder.league.setText(model.getLeague());
        holder.category.setText(model.getSportType());
        SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
        holder.commenceDate.setText(sdf.format(new Date(model.getCommenceTime())));
        holder.title1.setText(model.getHomeTeam());
        holder.title2.setText(model.getAwayTeam());
        if(model.getHomeTeamScore() != null){
            holder.homeTeamScore.setText(String.valueOf(model.getHomeTeamScore()));
        }else{
            holder.homeTeamScore.setText("0");
        }

        if(model.getAwayTeamScore() != null){
            holder.awayTeamScore.setText(String.valueOf(model.getAwayTeamScore()));
        }else{
            holder.awayTeamScore.setText("0");
        }


    }


    @Override
    public int getItemCount() {
        return eventArrayList.size();
    }

    public ArrayList<ModelEvent> getEventArrayList() {
        return eventArrayList;
    }





    public class HolderEventFinished extends RecyclerView.ViewHolder{

        TextView category, commenceDate, league, title1, title2, homeTeamScore, awayTeamScore;
        public HolderEventFinished(@NonNull View itemView) {
            super(itemView);
            category = itemView.findViewById(R.id.match_result_category);
            commenceDate = itemView.findViewById(R.id.match_result_playdate);
            league = itemView.findViewById(R.id.match_result_league);
            title1 = itemView.findViewById(R.id.match_result_homeTeam);
            title2 = itemView.findViewById(R.id.match_result_awayTeam);
            homeTeamScore = itemView.findViewById(R.id.match_result_homeScore);
            awayTeamScore = itemView.findViewById(R.id.match_result_awayScore);
        }

        public TextView getCategory() {
            return category;
        }

        public void setCategory(TextView category) {
            this.category = category;
        }

        public TextView getCommenceDate() {
            return commenceDate;
        }

        public void setCommenceDate(TextView commenceDate) {
            this.commenceDate = commenceDate;
        }

        public TextView getLeague() {
            return league;
        }

        public void setLeague(TextView league) {
            this.league = league;
        }

        public TextView getTitle1() {
            return title1;
        }

        public void setTitle1(TextView title1) {
            this.title1 = title1;
        }

        public TextView getTitle2() {
            return title2;
        }

        public void setTitle2(TextView title2) {
            this.title2 = title2;
        }

        public TextView getHomeTeamScore() {
            return homeTeamScore;
        }

        public void setHomeTeamScore(TextView homeTeamScore) {
            this.homeTeamScore = homeTeamScore;
        }

        public TextView getAwayTeamScore() {
            return awayTeamScore;
        }

        public void setAwayTeamScore(TextView awayTeamScore) {
            this.awayTeamScore = awayTeamScore;
        }

        @Override
        public String toString() {
            return "HolderEventFinished{" +
                    "category=" + category +
                    ", commenceDate=" + commenceDate +
                    ", league=" + league +
                    ", title1=" + title1 +
                    ", title2=" + title2 +
                    ", homeTeamScore=" + homeTeamScore +
                    ", awayTeamScore=" + awayTeamScore +
                    '}';
        }
    }
}
