package com.example.thomasbet.customDataTypes.strategy;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.InputType;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.thomasbet.MainActivity;
import com.example.thomasbet.R;
import com.example.thomasbet.RestClients.StrategyRestClient;
import com.example.thomasbet.RestClients.UserRestClient;
import com.example.thomasbet.StrategyFragment;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import android.widget.Toast;

public class AdapterStrategy extends RecyclerView.Adapter<AdapterStrategy.HolderPost> {
    private Context context;
    private MainActivity mainActivity;
    private StrategyFragment currentFragment;
    private ArrayList<ModelStrategy> postArrayList = new ArrayList<>();

    public AdapterStrategy(Context context, MainActivity mainActivity, StrategyFragment strategyFragment) {
        this.context = context;
        this.mainActivity = mainActivity;
        this.currentFragment = strategyFragment;
    }

    @NonNull
    @Override
    public HolderPost onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.strategybody, parent, false);
        return new HolderPost(view);
    }

    public void addModelPost(ModelStrategy modelPost){
        this.postArrayList.add(modelPost);
    }

    public void removeModels(){
        postArrayList.clear();
    }

    @Override
    public void onBindViewHolder(@NonNull HolderPost holder, int position) {
        ModelStrategy model = postArrayList.get(position);

        Executor executor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        executor.execute(new Runnable() {
            @Override
            public void run() {
                try {
                    model.setUser(Objects.requireNonNull(UserRestClient.GET_USER_BY_ID(model.getUserId())));
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");

                            holder.strategyPublisher.setText(model.getUser().getNickname());
                            holder.createdIn.setText(sdf.format(new Date(model.getDateNew())));
                            holder.editedAt.setText(sdf.format(new Date(model.getDateEdit())));
                            if(mainActivity.getUser().getRole_id() == 1 || model.getUserId() == mainActivity.getUser().getId()){
                                holder.buttonsLinearLayout.setVisibility(View.VISIBLE);
                                holder.strategyEdit.setOnClickListener(click -> {
                                    holder.setEditMode();
                                });
                                holder.strategySave.setOnClickListener(click -> {
                                    holder.sendUpdateRequest(holder.mapHolderPostToModelPost(holder), mainActivity, currentFragment);
                                    holder.setViewMode(mainActivity);

                                });
                                holder.strategyDelete.setOnClickListener(click -> {
                                    currentFragment.deleteStrategy(model.getStrategyId());
                                });
                            }else{
                                holder.buttonsLinearLayout.setVisibility(View.GONE);
                            }
                        }
                    });
                } catch (Exception e) {
                    System.out.println("ERROR ON USER GETNICKNAME");
                }
            }
        });


        holder.title.setText(model.getTitle());
        holder.strategyBody.setText(model.getBody());
        holder.dateEdit = model.getDateEdit();
        holder.dateNew = model.getDateNew();
        holder.strategyId = model.getStrategyId();
        holder.userId = model.getUserId();
        //holder.editedAt.setText("Upraveno v " + new Date(model.getDateEdit));

    }

    private void deleteStrategy(){

    }


    @Override
    public int getItemCount() {
        return postArrayList.size();
    }

    public ArrayList<ModelStrategy> getPostArrayList() {
        return postArrayList;
    }

    static class HolderPost extends RecyclerView.ViewHolder{
        Button moreBtn;
        EditText title;
        Long dateEdit;
        Long dateNew;
        Integer userId;
        Long blogId;
        Long strategyId;
        EditText strategyBody,strategyPublisher, createdIn, editedAt;
        ImageView strategyImage;
        Button strategyEdit;
        Button strategyDelete;
        Button strategySave;

        LinearLayout buttonsLinearLayout;
        public HolderPost(@NonNull View itemView) {
            super(itemView);
            moreBtn = itemView.findViewById(R.id.strategy_loadmore_btn);
            title = itemView.findViewById(R.id.strategy_title);
            //strategyImage = itemView.findViewById(R.id.strategy_image);
            strategyBody = itemView.findViewById(R.id.strategy_body);
            strategyPublisher = itemView.findViewById(R.id.strategy_publisher);
            strategyPublisher.setInputType(InputType.TYPE_NULL);
            strategyEdit = itemView.findViewById(R.id.strategy_article_edit);
            strategyDelete = itemView.findViewById(R.id.strategy_article_delete);
            buttonsLinearLayout = itemView.findViewById(R.id.strategy_articleControl_buttons);
            createdIn = itemView.findViewById(R.id.strategy_createdIn);
            createdIn.setInputType(InputType.TYPE_NULL);
            editedAt = itemView.findViewById(R.id.strategy_editedIn);
            editedAt.setInputType(InputType.TYPE_NULL);
            buttonsLinearLayout.setVisibility(View.GONE);

            strategySave = itemView.findViewById(R.id.strategy_article_save);
            strategySave.setVisibility(View.GONE);
        }

        private void setEditMode(){
            this.strategyEdit.setVisibility(View.GONE);
            this.strategySave.setVisibility(View.VISIBLE);


            title.setInputType(InputType.TYPE_CLASS_TEXT);
            strategyBody.setInputType(InputType.TYPE_CLASS_TEXT);
            strategyPublisher.setInputType(InputType.TYPE_NULL);
        }

        private void setViewMode(MainActivity mainActivity){

            this.strategySave.setVisibility(View.GONE);
            this.strategyEdit.setVisibility(View.VISIBLE);

            title.setInputType(InputType.TYPE_NULL);
            strategyBody.setInputType(InputType.TYPE_NULL);
        }


        private ModelStrategy mapHolderPostToModelPost(HolderPost holderPost){
            ModelStrategy modelPost = new ModelStrategy();
            modelPost.setUserId(holderPost.userId);
            modelPost.setStrategyId(holderPost.strategyId);
            modelPost.setBody(holderPost.strategyBody.getText().toString());
            modelPost.setTitle(holderPost.title.getText().toString());
            modelPost.setDateNew(dateNew);
            modelPost.setDateEdit(System.currentTimeMillis());

            return modelPost;
        }


        private void sendUpdateRequest(ModelStrategy modelPost, MainActivity mainActivity, StrategyFragment strategyFragment){
            Executor executor = Executors.newSingleThreadExecutor();
            Handler handler = new Handler(Looper.getMainLooper());

            executor.execute(new Runnable() {
                @Override
                public void run() {
                    try {
                        boolean result = StrategyRestClient.UPDATE_STRATEGY(modelPost);
                        handler.post(new Runnable() {
                            @Override
                            public void run() {
                                if(result){
                                    Toast.makeText(mainActivity, "Strategy has been updated", Toast.LENGTH_SHORT).show();
                                    strategyFragment.refresh();
                                }else{
                                    Toast.makeText(mainActivity, "Something went wrong", Toast.LENGTH_SHORT).show();
                                }

                            }
                        });
                    } catch (Exception e) {
                        System.out.println(e);
                    }
                }
            });
        }

    }
}
